from .models.lge.spec_l import ModelScraper_l as Specscraper_l
from .models.sony.spec_s import ModelScraper_s as Specscraper_s
from .models.sony.sepc_sjp import ModelScraper_sjp as Specscraper_sjp
from .models.pana.spec_p import ModelScraper_p as Specscraper_p
from .models.pana.spec_pjp import ModelScraper_pjp as Specscraper_pjp

from market_research.scraper.models.sony.visualizer_s import Visualizer_s
from .rtings.rtings import Rtings
from .rtings.rvisualizer import Rvisualizer
from .rtings.rurlsearcher import Rurlsearcher